// Использование сокетов на примере скачивания страницы html по протоколу HTTP.
// Версия 0.1

#include <stdio.h> // printf
#include <string.h>    // strlen
#include <unistd.h> // close
//#include <sys/socket.h>    // Возможно иногда нужно.
#include <arpa/inet.h> // TCP/IP: AF_INET, socket, inet_addr, connect, recv, send

int main(int argc , char *argv[])
{
	// Создаем сокет, т.е. сетевое подключение.
	int s = socket(AF_INET, // IPv4
		SOCK_STREAM, // потоковый, т.е. TCP
		0 /* протокол по умолчанию  */);

	// Проверяем успешность создания сокета.
	if( s == -1 )
	{
		printf("Сокет не удалось создать.");
		return -1;
	}

	printf("Сокет создан.\n");

	// Структура для указания адреса сервера.
	struct sockaddr_in srv;
	memset( &srv, 0, sizeof( srv ) );

	// Строчка содержащая ip адрес.
	const char *ip_str =
		// Адрес сервера машинноезрение.рф:
		"178.140.230.40";
		// Локальный адрес:
		//"127.0.0.1";
	srv.sin_family = AF_INET; // Для IPv4.
	srv.sin_addr.s_addr = inet_addr( ip_str );
	srv.sin_port = htons( 80 ); // Порт отвечающий за HTTP.

	// Подключаемся к серверу.
	if( connect(s, (struct sockaddr *)&srv, sizeof(srv) ) < 0 )
	{
		printf("Не удалось подключится к серверу.\n");
		return -1;
	}

	// Соединение успешно выполнено.
	// Переменая s содержит дискриптор соединения.
	// Далее можно делать, что хотим

	// У нас по плану выолнить запрос страницы html.
	// Детали как устроен сам запрос не будем обсуждать. Ограничимся только примером.
	// Он загружает страницу test_socket.html с сервера машинноезрение.рф.
	// Таким образом грузится страница: машинноезрение.рф/test_socket.html.
	// xn--80akaaied0aladi2a9h.xn--p1ai это перевод кирилического названия в латиницу.
	const char *msg = "GET /test_socket.html HTTP/1.1\r\nHost: xn--80akaaied0aladi2a9h.xn--p1ai\r\n\r\n";
	int msg_len = strlen( msg ); // Длина сообщения.

	// Отсылаем запрос.
	if( send(s, msg, msg_len, 0) < 0)
	{
		printf("Запрос не удался.\n");
		return -1;
	}

	int ret;
	bool gotit = false;

	// Получаем ответ от сервера.
	char reply[ 2001 ];
	while( ( ret = recv(s, reply, 2000, 0) ) > 0 )
	{
		gotit = true;// Хотя бы один раз ответ от сервера получили.
		reply[ret] = 0;// Создаем нуль строку.
		printf("%s", reply);
		//puts( reply );
		//printf(" %d\n ", ret);
		if( ret == 0 )
			break;
	}

	if( ret == -1 || gotit )
	{
		printf("error %d\n", ret);
	}

	// закрываем (освобождаем) сокет.
	close( s );
	return 0;
}

